#
# globals.tcl - global variables of map editor
#
# Copyright (C) 2005  Toni Ronkko
#
# Peach Gardens is free software; you can redistribute it and/or modify 
# it under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# Peach Gardens is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
# Nov 2004, Toni Ronkko <toni at users.playstation2-linux.com>
#
# $Id: globals.tcl,v 1.6 2005/01/16 23:33:58 tronkko Exp $
#
# $Log: globals.tcl,v $
# Revision 1.6  2005/01/16 23:33:58  tronkko
# definition of chip image was changed
#
# Revision 1.5  2005/01/09 20:58:23  tronkko
# accessibility, depth coordinate and background chip
#
# Revision 1.4  2004/12/09 22:10:15  tronkko
# region, file names
#
# Revision 1.3  2004/12/01 20:11:35  tronkko
# clarified the difference between palette and workspace by renaming
# some variables
#
# Revision 1.2  2004/11/30 20:30:52  tronkko
# source palette
#
# Revision 1.1  2004/11/28 11:08:16  tronkko
# Initial revision
#
#
#


### fonts ###

# font using in accessability buttons
set access_font "Helvetica 12 bold"

# font used in region names
set region_font "Helvetica 8 bold"


### widget names ###

# name of the statusbar widget
set statusbar_widget ""

# name of view widget on the left
set selector_widget ""

# name of editor widget on the right
set editor_widget ""

# name of menu entry that contains list of palettes
set palette_menu ""

# name of menu entry that contains list of maps
set map_menu ""

# pathname of top level menu
set menubar ""


### chips ###

# do we have the chipset loaded?
set chipset_present 0

# width and height of a chip
set chipw 0
set chiph 0

# number of chips in a chipset
set nchips 0

# Images of chips, array of (chip,level).  Level 0 contains original image
# while level 1 is a combined image with transparent areas filled with
# either background chip or a checkerboard pattern.  Moreover, level 1
# image is enlarged or shrinked according to current zoom level.
set chip_image(0,0) 0

# postion of chip within image (multiply with width & height to
# get pixel coordinate)
set chip_x1(0) 0
set chip_y1(0) 0

# depth coordinate
set chip_depth(0) 0

# bacground chip
set chip_bg(0) 0

# Accessability of regions.  Array of (chip,region) where region is between
# 1 and 9, inclusively.
set chip_terrain(0,0) 0


### workspaces ###

# workspace names given by the user
set workspace_name(0) ""

# is workspace used?  
set workspace_used(0) 0

# type of workspace: palette/map
set workspace_type(0) ""

# number of workspaces available
set nworkspaces 0

# chips in workspace, array of (workspace,x,y)
set workspace_chips(0,0,0) 0

# current workspace, array of (widget)
set cur_workspace(.) 0

# source workspace for copying between workspaces
set src_workspace 0

# width and height of workspace
set workspace_width(0) 0
set workspace_height(0) 0

# view coordinates of workspaces
set workspace_xview(0) 0
set workspace_yview(0) 0


### regions ###

# number of regions in a workspace, (workspace)
set workspace_nregions(0) 0

# location of region, array of (workspace,region)
set workspace_region_x(0,0) -1
set workspace_region_y(0,0) -1
set workspace_region_width(0,0) -1
set workspace_region_height(0,0) -1

# name of region, array of (workspace,region)
set workspace_region_name(0,0) ""

# is the region allocated, array of (workspace,region)
set workspace_region_used(0,0) 0


### view widget ###

# currently selected chips
set view_x1 -1
set view_y1 -1
set view_x2 -1
set view_y2 -1

# Global zoom level of view widgets, index to zoom_levels list.  Use
# set_zoom to change this.
set zoom 2

# mouse operation (select/draw/move/pan/edit)
set mouse_mode "select"

# path name of view widget containing selection
set selection_owner ""

# width of border between chips, array of (widget)
set view_border(.) 1


### undo ###

# location of saved chip
set undo_x(0) -1
set undo_y(0) -1

# chip stored to undo buffer or edit_status for undo limit
set undo_chip(0) 0

# workspace containing chip or -1 for undo limit
set undo_workspace(0) -1

# offset to undo buffer
set undo_start 0
set undo_ptr   1


### progam state ###

# protect workspace from editing, array of (widget)
set write_protect(.) 0

# has anything been edited since last save?
set edit_status 0

# has the title been updated to reflect edit_status
set window_status 0

# image file containing chips
set chipset_fn ""

# border width of chips in editor widget
set editor_grid 0

# current map file
set cur_file ""

# default directory in file dialogs
set default_dir "."

# geometry of main window from command line
set geometry ""
