#
# help.tcl - on-line help for map editor
# 
# Copyright (C) 2005  Toni Ronkko
# 
# Peach Gardens is free software; you can redistribute it and/or modify 
# it under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# Peach Gardens is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
# Dec, 2004  Toni Ronkko <toni at users.playstation2-linux.com>
# 
# $Id: help.tcl,v 1.3 2005/01/16 23:37:41 tronkko Exp $
#
# $Log: help.tcl,v $
# Revision 1.3  2005/01/16 23:37:41  tronkko
# more info on preparing transparent images
#
# Revision 1.2  2005/01/09 20:53:19  tronkko
# transparency & accessibility
#
# Revision 1.1  2004/12/09 22:16:54  tronkko
# Initial revision
#
#
#

# page number
set help_page 0

# max number of links in a page
set max_links 10

# number of links used
set nlinks 0


#
# Show help window, and allow user to browse between pages.
#
proc help_window { {page ""} } {
  global window_title
  global help_page
  global max_links

  # switch to desired page
  if { $page != "" } {
    set help_page $page
  }
  
  # create help window
  set win .help
  catch {destroy $win}
  toplevel $win
  wm title $win "$window_title Help"
  wm iconname $win "Help"

  # button row
  frame $win.buttons
  button $win.buttons.dismiss -text Dismiss -command "destroy $win"
  menubutton $win.buttons.page -text "Section" -underline 0 \
      -direction below -menu $win.buttons.page.menu -relief raised \
      -indicatoron 1
  menu $win.buttons.page.menu -tearoff 0 
  $win.buttons.page.menu add radiobutton -label "Introduction" \
      -variable help_page -value 0 -command "update_help" 
  $win.buttons.page.menu add radiobutton -label "Palettes and Maps" \
      -variable help_page -value 4 -command "update_help"
  $win.buttons.page.menu add radiobutton -label "Chips" \
      -variable help_page -value 7 -command "update_help"
  $win.buttons.page.menu add radiobutton -label "Editing Modes" \
      -variable help_page -value 2 -command "update_help"
  $win.buttons.page.menu add radiobutton -label "Accessability" \
      -variable help_page -value 5 -command "update_help"
  $win.buttons.page.menu add radiobutton -label "Transparency" \
      -variable help_page -value 6 -command "update_help"
  $win.buttons.page.menu add radiobutton -label "Keyboard Shortcuts" \
      -variable help_page -value 1 -command "update_help"
  $win.buttons.page.menu add radiobutton -label "Copying" \
      -variable help_page -value 3 -command "update_help"
  # next free: 8

  # text area with scrollbar
  text $win.text -yscrollcommand "$win.scroll set" -setgrid false \
	-width 80 -height 24 -wrap word -state disabled -bg white 
  scrollbar $win.scroll -command "$win.text yview"

  # pack buttons
  pack $win.buttons.dismiss -side left -padx 2 
  pack $win.buttons.page -side left -padx 2
  pack $win.buttons -side top -fill x -padx 4 -pady 4

  # pack text
  pack $win.scroll -side right -fill y
  pack $win.text -expand yes -fill both

  # configure text styles
  $win.text tag configure h1 -font "Helvetica 16 bold" \
      -lmargin1 8 -lmargin2 8 -rmargin 8
  $win.text tag configure h2 -font "Helvetica 14 bold" \
      -lmargin1 8 -lmargin2 8 -rmargin 8
  $win.text tag configure h3 -font "Helvetica 12" \
      -lmargin1 8 -lmargin2 8 -rmargin 8
  $win.text tag configure normal -font "Helvetica 10" \
      -lmargin1 8 -lmargin2 8 -rmargin 8
  $win.text tag configure emph -font "Helvetica 10 bold" \
      -lmargin1 8 -lmargin2 8 -rmargin 8

  # define link styles
  for {set i 0} {$i < $max_links} {incr i} {
    $win.text tag configure "link$i" -font "Helvetica 10" \
        -lmargin1 8 -lmargin2 8 -rmargin 8 \
        -foreground "#0000ff" -underline on
    $win.text tag bind "link$i" <Enter> "link_enter $i"
    $win.text tag bind "link$i" <Leave> "link_leave $i"
  }

  # keyboard navigation
  wm protocol $win WM_DELETE_WINDOW "destroy $win"
  bind $win <Escape> "destroy $win"  
  bind $win <KeyPress-q> "destroy $win"  
  bind $win <KeyPress-space> "$win.text yview scroll 1 pages"
  bind $win <Up>    "$win.text yview scroll -1 units"
  bind $win <Down>  "$win.text yview scroll 1 units"
  bind $win <Prior> "$win.text yview scroll -1 pages"
  bind $win <Next>  "$win.text yview scroll 1 pages"
  
  # show initial page
  update_help
}


#
# Switch pages in help browser.
#
# Note that two successive space characters within help text look ugly
# sensibly when a line break occurs between the two spaces.  Therefore,
# be sure to end sentences by just a single space.
# 
proc update_help {} {
  global help_page
  global nlinks
  set win .help

  # erase existing text
  $win.text configure -state normal
  $win.text delete 0.0 end
  $win.text insert end "\n" normal

  # release links
  set nlinks 0
  $win.text configure -cursor left_ptr  

  # print page
  switch $help_page {
    0 {
###### introduction ########################################################
      $win.text insert end "Introduction\n" h1
      $win.text insert end \
{
Welcome to Peach Gardens Map Editor! The editor allows you to\
modify existing maps and create new maps to be later included in\
Peach Gardens. The editor also allows you to place portals\
and other special\
places. These special locations are visible to\
to programmers working on the game, who also write actions that are\
triggered as a result of player entering the special\
locations. Thus, in order to play the map you have\
created in the map editor, someone must program the actions that\
are attached to the special places you designate.

This very help browser allows you to browse help titles included\
with map editor. To see the available help titles,\
click on the "Section" button above.

} normal

      # link to next subject
      $win.text insert end "Next:  " normal
      insert_link "Palettes and Maps" 4
    }

    
    1 {
###### keyboard shortcuts ##################################################
      $win.text configure -tabs {3c left 4c}
      $win.text insert end "Keyboard Shortcuts\n" h1
      $win.text insert end \
{
The following table lists keyboard shortcuts and mouse functions that \
are available in map editor:

} normal

      $win.text insert end "Key\tFunction\n" h3
      $win.text insert end \
{S      	Selection mode: use left mouse button to select chips.
D       	Draw mode: use left mouse button to paste selection.
P       	Pan mode: pan view with left mouse button.
M       	Move mode: move selection by dragging it.
E       	Edit mode: edit chip properties.
L       	Lock or release palette against accidental editing.
F1      	Launch this very help browser.
F5      	Re-read chipset image.
Delete   	Delete selected chips.
Control+S	Save file.
Control+Q	Quit.
Control+P	Create new palette.
Control+M	Create new map.
Control+L	Load file.
Control+Z	Undo
Cursor Keys 	Move selection.
Shift+Cursor	Expand selection.
Alt+P    	Open palette list. Type initial letter of palette to change.
Alt+M   	Open map list.
Shift+Left MB	Select chips, works in all modes.
Middle MButton	Paste selection, works in all modes.
Right MButton	Pan view, works in all modes.
Ctrl+Left MB	Move chips, works in all modes.
Ctrl+Right MB	Edit chip, works in all modes.

} normal

      # link to next subject
      $win.text insert end "Next:  " normal
      insert_link "Copying" 3
    }
    

    2 {
###### editing modes #######################################################
      $win.text insert end "Editing Modes\n" h1
      $win.text insert end \
{
Editing mode specifies the action taken by left mouse button. That\
is, the editing mode says what will happens when you left-click on a\
chip. There are five editing modes available:\
select, draw, pan, move and edit.

In } normal
      $win.text insert end "select mode" emph
      $win.text insert end \
{, left mouse button selects chips for drawing\
and various other operations. To enter selection mode, choose\
"select" from the edit menu or press S key. Then, move mouse\
cursor over a chip, and press left mouse button to select the\
chip. To select multiple chips, hold down left button and\
move the mouse pointer.

In } normal
      $win.text insert end "draw mode" emph
      $win.text insert end \
{, left mouse button pastes current selection to\
the location of mouse pointer. Note, however, that you can paste\
chips to both palette and map window. Palettes\
are typically edited at the beginning of the map creation process,\
and later used solely for drawing maps. If you do not\
intend to modify a palette, you can lock palettes to prevent \
accidental modification. To protect palettes, select "lock palettes"\
from edit menu or press L key. If the palette window\
is locked, the lock image at the lower right corner of the main\
window is closed. Otherwise the lock is open. When in locked mode,\
you cannot modify palettes. In fact, the draw mode is identical to\
the select mode, but only in the palette window. \
Protecting palettes from editing also\
allows you to select chips from palette window while in draw mode,\
and paste the chips to map without changing editing modes. This\
greatly reduces the need to switch editing modes all the\
time. Outside lock mode palettes can be edited just like\
maps.

} normal
      $win.text insert end "Move mode" emph
      $win.text insert end \
{ allows you to exchange two or more chips\
without permanently overwriting any of them. Move mode is\
especially useful for operating chips in palette window, as chips\
are typically tighly packed in a } normal
      insert_link "chipset image" 7
      $win.text insert end \
{, and you do not want to lose them. If the\
palette window is locked, move mode is identical to select mode.

} normal
      $win.text insert end "Pan mode" emph
      $win.text insert end \
{ allows you to move view by pressing down left mouse button\
and moving mouse pointer. To enter pan mode, select "pan" from the\
view menu or press P.

} normal
      $win.text insert end "Edit mode" emph
      $win.text insert end \
{ allows you to edit properties of chips. To enter edit mode,\
select edit from edit menu or press E key. Then, select a chip and\
a dialog window opens. For more information on editing chip\
properties, please see } normal

      insert_link "Accessibility" 5
      $win.text insert end " and " normal
      insert_link "Transparency" 6
      $win.text insert end \
{.

All the above functions can also be invoked by various shortcuts without\
changing editing modes. Please see } normal
      insert_link "Keyboard Shortcuts" 1
      $win.text insert end { \
for more information on the shortcuts.

} normal

      # link to next subject
      $win.text insert end "Next:  " normal
      insert_link "Accessibility" 5
    }


    3 {
###### copyright ###########################################################
      $win.text insert end "Copying Peach Gardens\n" h1
      $win.text insert end \
{
Copyright (C) 2005  Toni Ronkko

Peach Gardens is free software; you can redistribute it and/or modify\
it under the terms of the GNU General Public License as published by the\
Free Software Foundation; either version 2 of the License, or (at your\
option) any later version.

Peach Gardens is distributed in the hope that it will be useful, but\
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY\
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License\
for more details.

You should have received a copy of the GNU General Public License along\
with Peach Gardens; if not, write to the Free Software Foundation, Inc.,\
59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
} normal
    }

    4 {
###### copyright ###########################################################
      $win.text insert end "Palettes and Maps\n" h1
      $win.text insert end \
{
Map editor allows you to create palettes and maps. Palettes are shown\
on the left part of the main window while maps are shown on\
the right. Palettes exist solely to help you\
draw maps. You can to move\
and organize chips in palette window, and later paste selections from\
palette to a map. On the contrary to palettes, maps are visible in\
games.

Typically, there is a limited set of\
chips available for any game, and the chips must be tightly packed in\
} normal
      insert_link "chipset image" 7
      $win.text insert end \
{ in order to save space. This means that there cannot be many duplicate\
chips in a chipset. To solve the issue,\
map editor allows you to organize chips into palettes with\
distinct themes. You can also duplicate chips in palettes to\
prepare some commonly used patterns, for example. Palettes are not\
visible in the final game - they exist just to help you create maps.

Palettes are typically prepared at the beginning of map creation\
process. Once you have finished editing palettes, you can lock\
them against changes by pressing L or selecting "Lock Palettes"\
from the edit menu. Locking palettes means that you cannot\
move or paint new chips on the palette window, even if you try to.\
Instead, you can select chips in move and draw modes too. This\
allows you to select chips from the palette and paste them over the\
map without changing } normal
      insert_link "editing mode" 2
      $win.text insert end {.

On the contrary to palettes, maps shown on the right part of the main\
window can appear in games. However, they still need some\
programming in order to be played in a game. For example, each map\
needs an entry point, and some portals to other maps. The portals\
are only functional if they have been considered by programmers.

} normal

      # link to next subject
      $win.text insert end "Next:  " normal
      insert_link "Chips" 7
    }

    5 {
###### accessability #######################################################
      $win.text configure -tabs {3c left 4c}
      $win.text insert end "Accessibility\n" h1
      $win.text insert end \
{
Most games do not allow players to move completely freely in game\
area. Instead, the games devise obstacles such as walls and doorways\
to restrict players freedom. The obstacles also constitute to game\
play: the player must find ways to bypass obstacles. Besides denying\
movement, obstacles may also define when is the movement allowed. For\
example, rivers may be crossed only by a boat.

In Peach Gardens, accessibility is defined through\
chips that are further divided into 9 regions those\
accessibility is controlled separately. There are several accessibility\
modes available for regions:

} normal
      $win.text insert end "Mode\tFunction\n" h3
      $win.text insert end \
{Road    	Area with easy movement.
Grass   	Grassy area or floor with normal movement.
Forest  	Movement with great difficulty.
Water    	Body of water that can be accessed only by boat.
Mountain	Mountainous area that can be accessed only by birds.
Unreachable	Area that cannot be reached by any means.

Accessibility is defined in chip edit dialog. To access the dialog,\
press E to enter edit mode, and select a chip you would like to \
edit. In\
edit chip dialog, accessibility is indicated by initial\
letter. For example, G stands for grass and F for forest. To edit\
accessibility, move\
the mouse pointer over the region you want to edit and hold down third\
mouse button to see list of modes. (Alternatively, move the cursor with\
cursor keys and press space to open the menu.)  You can also move the \
mouse cursor\
over a region and press letters R, G, F, W, M or U to change\
the accessibility mode of that very region to road, grass, forest, water,\
mountain or unreachable, respectively. Moreover, pressing middle mouse\
button over a region rotates through accessibility modes and pressing\
left mouse button swaps between grass\
and unreachable modes.

Defining proper accessibility is a complex issue, especially for\
transparent chips. If you feel even a bit uncomfortable with\
accessibility, let the developers handle accessibility.

} normal

      # link to next subject
      $win.text insert end "Next:  " normal
      insert_link "Transparency" 6
    }

    
    6 {
###### transparency  #######################################################
      $win.text insert end "Transparency\n" h1
      $win.text insert end \
{
Peach Gardens is mainly two dimensional game, but every here and there\
you can see the player behind objects. For example, the\
player can be partially obscured by a chair that is at the front of\
him. This ability to get\
behind objects requires transparency. For the largest part\
transparency is defined by chipset creators and game developers\
outside the map\
editor. However, chip specific properties such as background\
chip and depth coordinate\
are assigned in map editor, often by game developers\
themselves. To edit these properties, press E to\
enter edit mode and select a chip you would like to\
edit. Wait until a window has popped up, and select transparency\
tab to edit depth coordinate and background chip.

} normal
      $win.text insert end "Depth coordinate" emph
      $win.text insert end \
{ helps define relative drawing order of chips and\
dynamic objects. Usually, the depth coordinate is set according to\
pixels at floor level. For a creature, location of a toe would\
specify the ideal depth coordinate. However, if a chip shows an\
image that does not\
touch floor at that region, the depth coordinate may refer to\
a pixel outside the boundaries of that very chip. This is useful\
with trees, for\
example. For a tree that spans multiple chips, the depth coordinate\
of the lowest part is set according to pixels at floor level. For\
chips above the root, depth coordinate is set in\
relation to the first chip. That is, depth coordinate of the\
root chip is added to the chip height.

Depth coordinate can be defined either by typing directly to\
respective field in edit chip dialog, or it can be selected by\
pointing image with a mouse. To set the coordinate using\
mouse, press left mouse button over the enlarged image,\
and a horizontal line will appear to mark the\
depth coordinate. (Note that the line is not updated if you edit the\
depth field directly.)

} normal
      $win.text insert end "Background chip" emph
      $win.text insert end \
{ is a chip that is always drawn below any\
objects. Every transparent chip must have a background chip. Usually,\
background chips show nothing but plain floor.

To specify background chip, open edit chip dialog as\
usual. Then, while the dialog window is still open and the mouse is\
in edit mode, locate the\
background chip of your interest, and press left mouse\
button to tranfer a chip identifier to edit chip dialog.

Alternatively, locate an identifier of a background chip of\
your interest, and insert the identifier directly to respective\
field in edit chip dialog. To find an\
identifier of a chip, press S to enter selection mode and select\
chip of your interest. Once you have selected a chip, you will see\
a message at status bar. This messages contains a chip identifier\
that can be used as a background chip. For example, message\
"Chip #81 at 0,5" means that you have selected chip number\
81 at map location 0,5. In this case the number 81 is\
the chip identifier you can use as a background \
chip.

Transparency is a complex issue. If you feel uncomfortable with\
transparency, leave the definition of transparent chips to developers.

} normal

      # link to next subject
      $win.text insert end "Next:  " normal
      insert_link "Keyboard Shortcuts" 1
    }


    7 {
###### chips ###############################################################
      $win.text configure -tabs {1c left}
      $win.text insert end "Chips\n" h1
      $win.text insert end \
{
Palettes and maps are constructed from small chips that are\
duplicated and organized in map editor. These chips are created in\
a graphics editor, and saved into a regular gif or pcx image\
file. This image file\
is then imported into map editor, and divided into evenly spaced\
chips.

To start working on a new chipset, select new from file menu, and locate\
a chipset image. Next, specify chip width and height, after which you\
will be asked to create an initial map. Specify name and size for this\
initial map, and press OK to start drawing a new map.

You can edit chipset image after you have created\
some maps to modify looks of some chips or to create new\
chips. The changes you make to chipset image\
will be visible in map editor once\
you reload the image from file menu or restart map\
editor. If you cannot find a chip from the existing\
palettes after having inserted new chips to chipset image, be sure\
to reset a palette from palette menu. Also note that you must\
not change the order of chips in chipset\
image or else the maps you have created may become erratic. If\
you need to re-arrange chips, you can do that safely in the palette\
window -- only use image editor for inserting new chips and modifying\
the existing ones.

Chips may have transparent areas. For example, doorways and windows of\
a house could be marked transparent to allow player to see through\
them. These transparent areas are designated in image editor by\
painting transparent pixels with violet (color index #ff00ff). Pixels\
painted with this color value are treated as transparent in Peach\
Gardens. Peach Gardens does not use other transparency information,\
which may be present on some file formats.

You can draw chipsets yourself, or you can download chipsets from\
the internet. The following sites contains huge\
amounts of chipsets for download:

	http://svatopluk.com/rm2k/material/
	http://rtgamer.free.fr/
	http://charas-project.net
	http://www.reinerstileset.4players.de:1059/

Please recall that images must be converted to gif or pcx file\
format in order to use them in map editor.

} normal

      # link to next subject
      $win.text insert end "Next:  " normal
      insert_link "Editing Modes" 2
    }

   
    default {
###### error page ##########################################################
      # keys
      $win.text insert end "Page not available!\n" h1
    }
  }

  # lock text
  $win.text configure -state disabled
}



#
# Insert hypertext link to a page.
#
proc insert_link { title page } {
  global help_page
  global nlinks
  global max_links
  set win .help

  # get link id
  set i $nlinks
  if { $i < $max_links } {
    # insert link and bind left mouse button
    $win.text insert end $title "link$i"
    $win.text tag bind "link$i" <1> "set help_page $page; update_help"    
    incr nlinks
  } else {
    modal_dialog "Error" "Too many links in a page" error    
  }
}


#
# Mouse pointer enters link.
#
proc link_enter { i } {
  set win .help
  $win.text tag configure "link$i" -foreground red
  $win.text configure -cursor hand2
}


#
# Mouse pointer leaves link.
#
proc link_leave { i } {
  set win .help
  $win.text tag configure "link$i" -foreground #0000ff
  $win.text configure -cursor left_ptr  
}
