#
# workspc.tcl - create, edit and delete workspaces
# 
# Copyright (C) 2005  Toni Ronkko
#
# Peach Gardens is free software; you can redistribute it and/or modify 
# it under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# Peach Gardens is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
# Nov 2004, Toni Ronkko <toni at users.playstation2-linux.com>
#
# $Id: workspc.tcl,v 1.4 2005/01/09 20:56:33 tronkko Exp $
#
# $Log: workspc.tcl,v $
# Revision 1.4  2005/01/09 20:56:33  tronkko
# do not create dummy region
#
# Revision 1.3  2004/12/09 22:14:54  tronkko
# moved out all code relating to user interface, resize
#
# Revision 1.2  2004/12/01 20:11:20  tronkko
# clarified the difference between palette and workspace
#
# Revision 1.1  2004/11/30 20:24:23  tronkko
# Initial revision
#
#


#
# Insert void workspace, and return the number of the new workspace.
# Note that the new workspace is not brought to view by default.
#
proc new_workspace { type name w h } {
  global nworkspaces
  global workspace_name
  global workspace_chips
  global workspace_width
  global workspace_height
  global workspace_xview
  global workspace_yview
  global workspace_used
  global workspace_type
  global workspace_nregions
  global workspace_region_used

  # get id for the new workspace
  set i -1
  set j 0
  while { $j < $nworkspaces } {
    if { ! $workspace_used($j) } {
      # found a free workspace
      set i $j
      break
    }
    incr j
  }
  if { $i < 0 } {
    # create one more workspace
    set i $nworkspaces
    incr nworkspaces
  }

  # initialize new workspace
  set workspace_name($i) $name
  set workspace_used($i) 1
  set workspace_type($i) $type
  set workspace_width($i) $w
  set workspace_height($i) $h

  # reset regions
  set workspace_nregions($i) 0
  set workspace_region_used($i,0) 0

  # fill workspace with void chips
  for {set y 0} {$y < $h} {incr y} {
    for {set x 0} {$x < $w} {incr x} {
      set workspace_chips($i,$x,$y) 0
    }
  }

  # initial view
  set workspace_xview($i) 0
  set workspace_yview($i) 0

  # return the identifier to the caller so that the view can be changed
  return $i
}


#
# Erase ith workspace.  Returns an identifier of some other workspace
# with identical type.
#
# Note that the function does not care to decrease nworkspaces variable.
# 
proc delete_workspace { i } {
  global nworkspaces
  global workspace_name
  global workspace_used
  global workspace_type
  global workspace_width
  global workspace_height
  global workspace_xview
  global workspace_yview
  global workspace_chips
  global workspace_nregions
  global workspace_region_used
  
  # remember type for a while
  set t $workspace_type($i)

  # fill workspace with void chips
  for {set y 0} {$y < $workspace_height($i)} {incr y} {
    for {set x 0} {$x < $workspace_width($i)} {incr x} {
      set workspace_chips($i,$x,$y) 0
    }
  }

  # erase regions
  set workspace_nregions($i) 0
  set workspace_region_used($i,0) 0

  # release workspace
  set workspace_name($i) ""
  set workspace_used($i) 0
  set workspace_type($i) "void"
  set workspace_width($i) -1
  set workspace_height($i) -1
  set workspace_xview($i) 0
  set workspace_yview($i) 0

  # find a workspace of type t
  for {set i 0} {$i < $nworkspaces} {incr i} {
    if { $workspace_used($i)  &&  $workspace_type($i) == $t } {
      # found a workspace with the very same type
      return $i
    }
  }

  # no workspaces of type t
  return -1
}


#
# Destroy all existing workspaces.  This is used before loading workspaces
# from disk.
#
proc destroy_workspaces {} {
  global nworkspaces
  global workspace_name
  global workspace_chips
  global workspace_width
  global workspace_height
  global workspace_xview
  global workspace_yview
  global workspace_used
  global workspace_type

  for {set i 0} {$i < $nworkspaces} {incr i} {
    delete_workspace $i
  }

  # there are no workspaces in memory
  set nworkspaces 0  
}



#
# Resize workspace with inserting and deleting chips as needed.
#
proc resize_workspace { cur w h } {
  global workspace_chips
  global workspace_width
  global workspace_height
  global workspace_xview
  global workspace_yview

  # reset view location if changing size
  if { $workspace_width($cur) != $w  ||  $workspace_height($cur) != $h } {
    set workspace_xview($cur) 0
    set workspace_yview($cur) 0
  }

  # enlarge width
  set i $workspace_width($cur)
  while { $i < $w } {
    for {set j 0} {$j < $workspace_height($cur)} {incr j} {
      set workspace_chips($cur,$i,$j) 0
    }
    incr i
  }
  set workspace_width($cur)  $w

  # enlarge height
  set j $workspace_height($cur)
  while { $j < $h } {
    for {set i 0} {$i < $workspace_width($cur)} {incr i} {
      set workspace_chips($cur,$i,$j) 0
    }
    incr j
  }        
  set workspace_height($cur) $h
}
